-- Lightroom SDK
local LrLogger 		= import "LrLogger"
local LrDialogs		= import "LrDialogs"
local LrFileUtils	= import "LrFileUtils"
local LrApplication	= import "LrApplication"
local LrTasks		= import 'LrTasks'
local LrPathUtils	= import 'LrPathUtils'
local LrStringUtils	= import 'LrStringUtils'
local LrDate		= import 'LrDate'

require 'PluginData'
require 'startupDialog'

local pathToPhotoMap

function tryAccessCatalog(fn, ...)
  local status, data = LrTasks.pcall(fn, unpack(arg))
  if status == true then
    return data
  else
    logger:trace("Cannot retrieve photos from the Lightroom catalog: " .. data)
    error("Cannot retrieve data from the Lightroom catalog. The catalog might be corrupted.")
  end
end

function getSinglePhotoPathFromCatalog(catalog, photoPathLower)
  local map = {}
  local photosInCatalog = catalog:getAllPhotos()
  for i,photoInCatalog in ipairs(photosInCatalog) do
    local photoPathInCatalog = photoInCatalog:getRawMetadata("path")
    if photoPathInCatalog ~= nil then
      local photoPathInCatalogLower = photoPathInCatalog:lower()
      if photoPathLower == photoPathInCatalogLower then
        map[photoPathInCatalogLower] = photoInCatalog
        return map
      end
    end
  end
  return map
end

function getPhotoPathsFromCatalog(catalog)
  local map = {}
  local photosInCatalog = catalog:getAllPhotos()
  for i,photoInCatalog in ipairs(photosInCatalog) do
    local photoPathInCatalog = photoInCatalog:getRawMetadata("path")
    if photoPathInCatalog ~= nil then
      map[photoPathInCatalog:lower()] = photoInCatalog
    end
  end
  return map
end

function findPhotoByPathInCatalog(catalog, photoPath, fastImport)
  local photo = catalog:findPhotoByPath(photoPath)
  if photo ~= nil then
    return photo
  end
  local photoPathLower = photoPath:lower()
  if fastImport == true then
    pathToPhotoMap = tryAccessCatalog(getSinglePhotoPathFromCatalog, catalog, photoPathLower)
    return pathToPhotoMap[photoPathLower]
  end
  if pathToPhotoMap == nil then
    pathToPhotoMap = tryAccessCatalog(getPhotoPathsFromCatalog, catalog)
  end
  for path,photo in pairs(pathToPhotoMap) do
    if photoPathLower == path then
      return photo
    end
  end
end

function addPhotosToNewCollection(catalog, importedPhotos)
  local currentTime = LrDate.currentTime()
  local newCollectionName = LrDate.timeToUserFormat(currentTime, "%Y-%m-%d %H:%M:%S")

  local dxoCollection = nil
  local importedCollection = nil
  catalog:withWriteAccessDo("1",
    function(context)
      dxoCollection = catalog:createCollectionSet("DxO PhotoLab", nil, true)
    end
  )
  catalog:withWriteAccessDo("2",
    function(context)
      importedCollection = catalog:createCollection(newCollectionName, dxoCollection, true)
    end
  )
  logger:trace("importedCollection: " .. importedCollection:getName())
  catalog:withWriteAccessDo("3",
    function(context)
      importedCollection:addPhotos(importedPhotos)
    end
  )
  catalog:setActiveSources(importedCollection)
end

function auxImportFile(catalog, fileToImport, fileToStackWith, fastImport)
  local photoAdded = nil
  if nil ~= fileToImport then
    fileToImport = LrStringUtils.trimWhitespace(fileToImport)
    logger:trace("fileToImport: " .. fileToImport)
    if(LrFileUtils.exists(fileToImport)) then 
      photoAdded = findPhotoByPathInCatalog(catalog, fileToImport, fastImport)
      if nil == photoAdded then
        local photoToStackWith = nil
        if nil ~= fileToStackWith then
          photoToStackWith = nil
        end
        catalog:withWriteAccessDo("Import from Application",
          function(context)
            if nil ~= fileToStackWith then
              photoAdded = catalog:addPhoto(fileToImport, photoToStackWith)
              if nil ~= photoAdded then
                logger:trace("Photo added: " .. fileToImport .. ", fileToStackWith: " .. fileToStackWith)
              else
                logger:trace("Photo not added: " .. fileToImport .. ", fileToStackWith: " .. fileToStackWith)
              end
            else
              photoAdded = catalog:addPhoto(fileToImport)
              if nil ~= photoAdded then
                logger:trace("Photo added without stacking: " .. fileToImport)
              else
                logger:trace("Photo not added without stacking: " .. fileToImport)
              end
            end
          end
        )
      else
        logger:trace("last fileToImport has been imported before")
      end
    else
      logger:trace("last fileToImport does not exist")
    end
  else
    logger:trace("trying to import nil fileToImport in auxImportFile")
  end
  return photoAdded
end

function tryToImportFromFile(fileName)
  logger:trace("Test import started for file: " .. fileName)
  if LrFileUtils.exists(fileName) then
    LrTasks.startAsyncTask( 
      function(context)
        local importedFiles = {}
        local importedPhotos = {}
        local fileData = LrFileUtils.readFile(fileName)
        local fileDataLen = string.len(fileData)
        local stackLines = {}
        local _, numberOfLines = string.gsub(fileData, "\n", "\n")
        local fastImport = numberOfLines <= 1
        --for each line
        local lineFrom = 1
        local catalog = LrApplication.activeCatalog()
        repeat
          local lineTo = string.find(fileData, "\n", lineFrom)
          if lineTo ~= nil then
            logger:trace("lineFrom: " .. lineFrom .. ", lineTo: " .. lineTo)
            local currentLine = string.sub(fileData, lineFrom, lineTo)
            local firstChrVal = string.sub(currentLine, 1, 1) 
            logger:trace("firstChrVal: " .. firstChrVal)
            if firstChrVal == PluginData.dxoDoNotImportFileSignature then
                --do not import current file, only put it to stack
                local firstNotIntChrIndex = string.find(currentLine, " ")
                fileToStack = LrStringUtils.trimWhitespace(string.sub(currentLine, firstNotIntChrIndex+1))
                table.insert(stackLines, fileToStack)
                logger:trace("fileToStack: " .. fileToStack)
            else
              if (firstChrVal >= '0') and (firstChrVal <= '9') then
                --get stack index of line to stack
                local fileToStack = nil
                local fileToImport = nil
                --looking for " " after the index of file to stack
                local firstNotIntChrIndex = string.find(currentLine, " ")
                if nil ~= firstNotIntChrIndex then
                  local stackIndex = tonumber(string.sub(currentLine, 1, firstNotIntChrIndex))+1
                  fileToStack = stackLines[stackIndex]
                  fileToImport = string.sub(currentLine, firstNotIntChrIndex+1)
                  logger:trace("stackIndex: " .. stackIndex .. ",fileToStack: " .. fileToStack)
                end
                if nil ~= fileToImport then
                  local photoImported = auxImportFile(catalog, fileToImport, fileToStack, fastImport)
                  if nil ~= photoImported then
                    table.insert(importedFiles, fileToImport)
                    table.insert(importedPhotos, photoImported)
                  end
                end
              else
                local fileToImport = LrStringUtils.trimWhitespace(currentLine)
                table.insert(stackLines, fileToImport)
                local photoImported = auxImportFile(catalog, fileToImport, nil, fastImport)
                if nil ~= photoImported then
                  table.insert(importedFiles, fileToImport)
                  table.insert(importedPhotos, photoImported)
                end
              end
            end
            lineFrom = lineTo + 1
          end
        until(nil == lineTo)
        if #importedPhotos > 0 then
          addPhotosToNewCollection(catalog, importedPhotos)
          catalog:setSelectedPhotos(importedPhotos[1], importedPhotos)
        end
        LrFileUtils.delete(fileName)
        pathToPhotoMap = nil
      end
    )
  end
end

logger = LrLogger(PluginData.dxoLoggerFileName)
if PluginData.dxoUseLogging then
  logger:enable("logfile")
end

logger:trace("======= PluginInit Loaded " .. os.date() .. " =======")

showStartupDialogOnce()

local standardTempDirPath = LrPathUtils.getStandardFilePath('temp')
local importFileName = LrPathUtils.child(standardTempDirPath, PluginData.dxoTempImportFileName)
local triggerFileName = LrPathUtils.child(standardTempDirPath, PluginData.dxoTriggerFileName)

if LrFileUtils.exists(triggerFileName) then
  LrFileUtils.delete(triggerFileName)
  tryToImportFromFile(importFileName)
end

--g_isPluginRunning is used since async task can be running even after plug-in shutdown
g_isPluginRunning = 1

LrTasks.startAsyncTask( function()
  while g_isPluginRunning == 1 do
    if LrFileUtils.exists(triggerFileName) then
      LrFileUtils.delete(triggerFileName)
      tryToImportFromFile(importFileName)
    end
    --sleep for 1 second
    LrTasks.sleep(1)
  end
end)
